/* -*- mode: c; coding: utf-8 -*- */

#include <IceC/IceC.h>

typedef struct {
    Ice_Object __base;
} Robot_Motion;

typedef Robot_Motion* Robot_MotionPtr;

typedef Ice_ObjectPrx Robot_MotionPrx;
typedef Ice_ObjectPrx* Robot_MotionPrxPtr;

/* forward declaration */
static void
Robot_Motion_methodHandler(Robot_MotionPtr self,
                           Ice_InputStreamPtr is,
                           Ice_OutputStreamPtr os);

static void
Robot_Motion_init(Robot_MotionPtr self) {

    trace();
    
    Ice_Object_init((Ice_ObjectPtr)self,
            (Ice_MethodHandlerPtr)Robot_Motion_methodHandler);
}

void
Robot_MotionI_forward(Robot_MotionPtr self,
                      Ice_Int time) __attribute__((weak));

void
Robot_MotionI_backward(Robot_MotionPtr self,
                       Ice_Int time) __attribute__((weak));

static void
Robot_Motion_forward__dispatch(Robot_MotionPtr self,
                               Ice_InputStreamPtr is,
                               Ice_OutputStreamPtr os) {
    Ice_Int time = 0;

    trace();
    
    Ice_InputStream_readInt(is, &time);
    
    Robot_MotionI_forward(self, time);
}

static void
Robot_Motion_backward__dispatch(Robot_MotionPtr self,
                                Ice_InputStreamPtr is,
                                Ice_OutputStreamPtr os) {
    Ice_Int time = 0;

    trace();
    
    Ice_InputStream_readInt(is, &time);
    
    Robot_MotionI_backward(self, time);
}

static void
Robot_Motion_methodHandler(Robot_MotionPtr self,
                           Ice_InputStreamPtr is,
                           Ice_OutputStreamPtr os) {
    Ice_Int operationSize = 0;
    char operation[MAX_OPERATION_NAME_SIZE];
    byte mode = 0;
    byte contextSize = 0;
    Ice_Int encapSize = 0;
    byte major = 0;
    byte minor = 0;

    trace();
    
    Ptr_check(self);
    Ptr_check(is);
    Ptr_check(os);
    
    Ice_InputStream_readSize(is, &operationSize, false);
    assert(operationSize <= MAX_OPERATION_NAME_SIZE);
    
    Ice_InputStream_readString(is, operation);
    
    /* FIXME: operationMode ignored */
    Ice_InputStream_readByte(is, &mode);
    
    /* FIXME: Context not supported */
    Ice_InputStream_readByte(is, &contextSize);
    assert(contextSize == 0);
    
    Ice_InputStream_readInt(is, &encapSize);
    Ice_InputStream_readByte(is, &major);
    Ice_InputStream_readByte(is, &minor);
    assert(major == 1 && minor == 0);
    assert(encapSize - 6 == (is->size - (is->next - is->data)));
    
    if (strcmp(operation, "forward") == 0) {
        Robot_Motion_forward__dispatch(self, is, os);
        return;
    }
    
    if (strcmp(operation, "backward") == 0) {
        Robot_Motion_backward__dispatch(self, is, os);
        return;
    }
    
    c_throw("Ice_OperationNotExistException", __FILE__, __LINE__);
}

static void
Robot_Motion_forward(Robot_MotionPrxPtr self,
                     Ice_Int time) {
    Ice_OutputStreamPtr os = &(self->stream);
    bool ready = 0;

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    if (self->mode == Ice_ReferenceModeTwoway) {
        Ice_OutputStream_writeInt(os, 1);
    }
    if (self->mode != Ice_ReferenceModeTwoway) {
        Ice_OutputStream_writeInt(os, 0);
    }
    
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "forward");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeInt(os, time);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* parse reply message (only on twoway invocations) */
    if (self->mode == Ice_ReferenceModeTwoway) {
        Ice_InputStream is;
        char expected[10] = {'I', 'c', 'e', 'P', 1, 0, 1, 0, Ice_replyMsg, 0};
        Ice_Int size = 0;
        Ice_Int requestId = 0;
        Ice_Byte replyStatus = 0;
        Ice_Int encapsSize = 0;
        Ice_Byte skipByte = 0;
        
        IcePlugin_Connection_dataReady(&(self->connection), &ready);
        if (!ready) {
            /* FIXME: do something */
            return;
        }
        
        /* reply header */
        Ice_InputStream_init(&is, self->connection.fd, self->connection.epinfo->type);
        if (memcmp(expected, is.next, 10) != 0) {
            c_throw("Ice_ProtocolException", __FILE__, __LINE__);
            return;
        }
        
        /* skip header from input stream */
        is.next += 10;
        Ice_InputStream_readInt(&is, &size);
            if (size > MAX_MESSAGE_SIZE) {
            c_throw("Ice_MemoryLimitException", __FILE__, __LINE__);
            return;
        }
        
        /* reply body */
        Ice_InputStream_readInt(&is, &requestId);
            Ice_InputStream_readByte(&is, &replyStatus);
            
        /* FIXME: check replyStatus */
        
        Ice_InputStream_readInt(&is, &encapsSize);
            Ice_InputStream_readByte(&is, &skipByte);
            Ice_InputStream_readByte(&is, &skipByte);
        }
    
    Ice_ObjectPrx_closeConnection(self);
}

static void
Robot_Motion_backward(Robot_MotionPrxPtr self,
                      Ice_Int time) {
    Ice_OutputStreamPtr os = &(self->stream);
    bool ready = 0;

    trace();
    
    Ptr_check(self);
    
    Ice_ObjectPrx_connect(self);
    Ice_OutputStream_init(os);
    
    /* request header */
    Ice_OutputStream_writeBlob(os, Ice_header, sizeof(Ice_header));
    
    /* request body */
    if (self->mode == Ice_ReferenceModeTwoway) {
        Ice_OutputStream_writeInt(os, 1);
    }
    if (self->mode != Ice_ReferenceModeTwoway) {
        Ice_OutputStream_writeInt(os, 0);
    }
    
    Ice_OutputStream_writeIdentity(os, self->identity);
    Ice_OutputStream_writeString(os, "");
    Ice_OutputStream_writeString(os, "backward");
    Ice_OutputStream_writeByte(os, Ice_Normal);
    Ice_OutputStream_writeByte(os, 0);
    
    /* encapsulated params */
    Ice_OutputStream_startWriteEncaps(os);
    Ice_OutputStream_writeInt(os, time);
    Ice_OutputStream_endWriteEncaps(os);
    Ice_OutputStream_setMessageSize(os);
    
    Ice_ObjectPrx_send(self);
    
    /* parse reply message (only on twoway invocations) */
    if (self->mode == Ice_ReferenceModeTwoway) {
        Ice_InputStream is;
        char expected[10] = {'I', 'c', 'e', 'P', 1, 0, 1, 0, Ice_replyMsg, 0};
        Ice_Int size = 0;
        Ice_Int requestId = 0;
        Ice_Byte replyStatus = 0;
        Ice_Int encapsSize = 0;
        Ice_Byte skipByte = 0;
        
        IcePlugin_Connection_dataReady(&(self->connection), &ready);
        if (!ready) {
            /* FIXME: do something */
            return;
        }
        
        /* reply header */
        Ice_InputStream_init(&is, self->connection.fd, self->connection.epinfo->type);
        if (memcmp(expected, is.next, 10) != 0) {
            c_throw("Ice_ProtocolException", __FILE__, __LINE__);
            return;
        }
        
        /* skip header from input stream */
        is.next += 10;
        Ice_InputStream_readInt(&is, &size);
            if (size > MAX_MESSAGE_SIZE) {
            c_throw("Ice_MemoryLimitException", __FILE__, __LINE__);
            return;
        }
        
        /* reply body */
        Ice_InputStream_readInt(&is, &requestId);
            Ice_InputStream_readByte(&is, &replyStatus);
            
        /* FIXME: check replyStatus */
        
        Ice_InputStream_readInt(&is, &encapsSize);
            Ice_InputStream_readByte(&is, &skipByte);
            Ice_InputStream_readByte(&is, &skipByte);
        }
    
    Ice_ObjectPrx_closeConnection(self);
}

